/* 
 * algorithm.cpp - algorithm handling
 *
 * 2003  Benedikt Meurer <bmeurer@unix-ag.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <string.h>
#include <stdlib.h>

#include "algorithm.h"


Algorithm *
Algorithm::find(const char *identifier)
{
	const SortFunctionDescr *p;

	for (p = sortFunctions; p->fctn != 0; p++) {
		if (strcasecmp(p->identifier, identifier) == 0)
			return (new Algorithm(p));
	}

	cerr << "unknown sort algorithm \"" << identifier << '\"' << endl
	     << endl
	     << "Available algorithms:" << endl
	     << getAvailableAlgorithms() << endl;

	/* its easier to fail here, instead of returning a NULL pointer */
	exit(EXIT_FAILURE);

	/* NOT REACHED */
}

Algorithm::Algorithm(const SortFunctionDescr *descr)
{
	fctn = descr;
	stats = new Stats();
}

Algorithm::~Algorithm(void)
{
	delete stats;
}

void
Algorithm::run(const Array &a)
{
	/* to speed things up */
	static Array tmpa(1);

	tmpa = a;

	stats->use();

	fctn->fctn(tmpa);
	
	stats->unuse();
}



