/*
 * array.h - array handling
 *
 * 2003  Benedikt Meurer <bmeurer@unix-ag.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _ARRAY_H
#define _ARRAY_H

#include <sys/types.h>

#include <assert.h>
#include <iostream.h>

#include "item.h"


class Array
{
	public:
		/* initial sort order */
		enum Order {
			ASCEND,
			DESCEND,
			RANDOM
		};

	private:
		Item	*items;
		size_t	len;

		/* check if an item is unique in a given part of the array */
		bool unique(size_t, const Item &);

	public:
		Array(size_t);
		Array(const Array &);

		unsigned long length(void) const { return (len); }

		/* copy operators */
		Array &operator = (const Array &);
		Array &operator = (const Array *);

		Item &operator [] (size_t) const;

		/* reset the array with "a given order" */
		void reset(Order = Array::RANDOM);

		/* friends */
		friend void swap(Array &, size_t, size_t);
};

inline ostream &
operator << (ostream &os, const Array &a)
{
	size_t i;

	for (i = 0; i < a.length() - 1; i++)
		os << a[i] << ", ";

	return (os << a[i]);
}

inline istream &
operator >> (istream &is, Array &a)
{
	size_t i;

	for (i = 0; i < a.length(); i++)
		is >> a[i];

	return (is);
}

/* swap operation as introduced in EI1 */
inline void
swap(Array &a, size_t i, size_t j)
{
	assert(i < a.len);
	assert(j < a.len);

	swap(a.items[i], a.items[j]);
}

#endif	/* _ARRAY_H */
