/*
 * item.h - Data item class
 *
 * 2003  Benedikt Meurer <bmeurer@unix-ag.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _ITEM_H
#define _ITEM_H

#include <iostream.h>

#include "stats.h"


class Item
{
	private:
		long data;

	public:
		/* constructors */
		Item(int value)		{ data = value; }
		Item(void)		{ data = 0; }
		Item(const Item &);

		/* copy operators */
		Item &operator = (const Item &);
		Item &operator = (const Item *);

		/* cmp operators */
		bool operator == (const Item &) const;
		bool operator != (const Item &) const;
		bool operator <= (const Item &) const;
		bool operator >= (const Item &) const;
		bool operator <  (const Item &) const;
		bool operator >  (const Item &) const;

		/* cmp operators (with pointers) */
		bool operator == (const Item *) const;
		bool operator != (const Item *) const;
		bool operator <= (const Item *) const;
		bool operator >= (const Item *) const;
		bool operator <  (const Item *) const;
		bool operator >  (const Item *) const;

		/* generate a random Item in [1..n] */
		static Item random(long);

		/* swap two items */
		friend void swap(Item &, Item &);

		/* compare two items (like strcmp) */
		friend int cmp(const Item &, const Item &);

		/* iostream stuff */
		friend ostream &operator << (ostream &, const Item &);
		friend istream &operator >> (istream &, Item &);
};

/* swap two items */
inline void
swap(Item &item1, Item &item2)
{
	long tmp;

	/* statistics */
	Stats::addMovs(3);

	tmp = item1.data;
	item1.data = item2.data;
	item2.data = tmp;
}

/* compare two items (like strcmp) */
inline int
cmp(const Item &item1, const Item &item2)
{
	/* statistics */
	Stats::incCmps();

	if (item1.data < item2.data)
		return (-1);
	else if (item1.data > item2.data)
		return (1);
	else
		return (0);
}

inline ostream &
operator << (ostream &os, const Item &item)
{
	return (os << item.data);
}

inline istream &
operator >> (istream &is, Item &item)
{
	return (is >> item.data);
}

#endif	/* _ITEM_H */

