/*
 * main.cpp - main module
 *
 * 2003  Benedikt Meurer <bmeurer@unix-ag.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "array.h"
#include "sortfctns.h"
#include "stats.h"
#include "table.h"


static inline const char *
orderName(Array::Order order)
{
	switch (order) {
	case Array::RANDOM:
		return ("RANDOM");
	case Array::ASCEND:
		return ("ASCEND");
	case Array::DESCEND:
		return ("DESCEND");
	}
	return ("UNKNOWN");
}

int
main(int argc, char **argv)
{
	Array::Order order = Array::RANDOM;
	char *algorithms = getAvailableAlgorithms();
	int items = 10;
	int times = 10;
	int c;

	while ((c = getopt(argc, argv, "a:n:o:t:")) != -1) {
		switch (c) {
		case 'a':
			algorithms = optarg;
			break;

		case 'n':
			items = atoi(optarg);
			break;

		case 'o':
			if (strcasecmp(optarg, "random") == 0)
				order = Array::RANDOM;
			else if (strcasecmp(optarg, "ascend") == 0)
				order = Array::ASCEND;
			else if (strcasecmp(optarg, "descend") == 0)
				order = Array::DESCEND;
			else {
				cerr << optarg << ": unknown sort order"
				     << endl;
				exit(EXIT_FAILURE);
			}
			break;

		case 't':
			times = atoi(optarg);
			break;

		case 'h':
		default:
			cerr << "sortchk [-a <algorithms>] [-n <items>]"
			     << " [-t <times>]"
			     << endl;
			return (1);
		}
	}

	Table t(algorithms);

	cout << endl 
	     << "Running test:" << endl
	     << "-------------" << endl
	     << "Items: " << items << endl
	     << "Order: " << orderName(order) << endl
	     << "Times: " << times << endl
	     << endl;

	t.run(times, items, order);

	cout << t << endl;

	return (0);
}

