/*
 * stats.h - statistics module
 *
 * 2003  Benedikt Meurer <bmeurer@unix-ag.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _STATS_H
#define _STATS_H

#include <sys/types.h>

#include <assert.h>


class Stats
{
	public:
		u_int64_t ncmps;	/* comparison counter */
		u_int64_t nmovs;	/* data move counter */

	public:
		Stats(void) {ncmps = nmovs = 0;}

		void use(void)		{ current = this; }
		void unuse(void)	{ current = 0; }

	protected:
		/* currently active statistics */
		static Stats *current;

	public:
		static void addCmps(u_int64_t);
		static void addMovs(u_int64_t);
		static void decCmps(void);
		static void decMovs(void);
		static void incCmps(void);
		static void incMovs(void);
		static void subCmps(u_int64_t);
		static void subMovs(u_int64_t);

		u_int64_t cmps(void) { return (ncmps); }
		u_int64_t movs(void) { return (nmovs); }

		/* calculate per times */
		void cpt(u_int64_t);
};

inline void
Stats::cpt(u_int64_t times)
{
	assert(times != 0);

	ncmps /= times;
	nmovs /= times;
}

#endif	/* _STATS_H */

