/*
 * table.cpp - algorithm test result table
 *
 * 2003  Benedikt Meurer <bmeurer@unix-ag.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <iomanip.h>
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

#include "table.h"


Table::Table(const char *list)
{
	char *p, *sp = strdup(list);
	
	rows = 0;

	for (p = strtok(sp, ","); p != NULL; p = strtok(NULL, ",")) {
		if (rows < MAXALGOS)
			algos[rows++] = Algorithm::find(p);
		else {
			/* no more space left */
			break;
		}
	}

	free(sp);
}

void
Table::run(int times, size_t items, Array::Order order)
{
	Array a(items);

	a.reset(order);

	for (int i = 0; i < times; i++) {
		/* test all given function */
		for (int j = 0; j < rows; j++)
			algos[j]->run(a);


		/* generate a new random permutation */
		if (order == Array::RANDOM)
			a.reset(order);
	}

	/* recalculate the stats per times */
	for (int j = 0; j < rows; j++)
		algos[j]->cpt(times);
}

ostream &
operator << (ostream &os, const Table &tb)
{
	int maxnamelen = sizeof("Algorithm");
	int maxnumlen = 12;

	for (int j = 0; j < tb.rows; j++) {
		int len = strlen(tb.algos[j]->name());
		
		if (len > maxnamelen)
			maxnamelen = len;
	}

	os << setw(maxnamelen) << setiosflags(ios::left)
		<< "Algorithm" << " | "
	   << setw(maxnumlen) << setiosflags(ios::right) 
	   	<< "Comparisons" << " | "
	   << setw(maxnumlen) << setiosflags(ios::right)
	   	<< "Data moves" << endl;

	/* print divide line */
	for (int i = 0; i < maxnamelen; i++) os << "-"; os << "-+-";
	for (int i = 0; i < maxnumlen; i++) os << "-"; os << "-+-";
	for (int i = 0; i < maxnumlen; i++) os << "-"; os << endl;

	for (int j = 0; j < tb.rows; j++) {
		os << setw(maxnamelen) << tb.algos[j]->name()
			<< setw(3) << " | "
		   << setw(maxnumlen) << tb.algos[j]->cmps()
		   	<< setw(3) << " | "
		   << setw(maxnumlen) << tb.algos[j]->movs()
		   	<< endl;
	}

	return (os);
}



